/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.sys.startstopp.skriptvew;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;

import java.io.BufferedInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import org.jdom.Element;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * Klasse zum Testen der Klasse XMLTools
 * @author Dambach Werke GmbH
 */
public class XMLToolsTest
{

  /**
   * @throws java.lang.Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
    // Testumgebung initialisieren
    
    StartStoppApp a = new StartStoppApp ();
    
    a.setStartStoppKonfiguration( "C:\\JUnit");
    a.setStartStoppInkarnationsName( "StartStopp" );
    a.bestimmeRechnerAdresse();

  }

  /**
   * @throws java.lang.Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @Before
  public void setUp() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @After
  public void tearDown() throws Exception
  {
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.skriptvew.XMLTools#leseRootElement(java.lang.String)}.
   * Methode prft eine syntaktisch korrekte XML-Datei
   */
  @Test
  public void testLeseRootElementNormal()
  {
    String resource = "../xml/startStopp_Normal.xml";
    
    String fehler = pruefeXMLString( resource );
    assertNull( fehler );
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.skriptvew.XMLTools#leseRootElement(java.lang.String)}.
   * Methode prft eine fehlerhafte XML-Datei
   */
  @Test
  public void testLeseRootElementFehler()
  {
    String resource = "../xml/startStopp_Fehler.xml";
    
    String fehler = pruefeXMLString( resource );
    assertNotNull( fehler );
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.skriptvew.XMLTools#leseRootElement(java.lang.String)}.
   * Methode prft XML-Datei ohne USV Angabe
   */
  @Test
  public void testLeseRootElementOhneUSV()
  {
    String resource = "../xml/startStopp_ohneUSV.xml";
    
    String fehler = pruefeXMLString( resource );
    assertNull( fehler );
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.skriptvew.XMLTools#leseRootElement(java.lang.String)}.
   * Methode prft XML-Datei mit einer USV Angabe
   */
  @Test
  public void testLeseRootElementMitEinerUSV()
  {
    String resource = "../xml/startStopp_mitEinerUSV.xml";
    
    String fehler = pruefeXMLString( resource );
    assertNull( fehler );
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.skriptvew.XMLTools#leseRootElement(java.lang.String)}.
   * Methode prft XML-Datei mit zwei USV Angaben
   */
  @Test
  public void testLeseRootElementMitZweiUSV()
  {
    String resource = "../xml/startStopp_mitZweiUSV.xml";
    
    String fehler = pruefeXMLString( resource );
    assertNotNull( fehler );
  }

  /**
   * Methode zum Einlesen einer XML-Datei aus den Resourcen in einen String
   * @param resourceName Name der Resource
   * @return String mit XML-Struktur
   */
  public String einlesenAusResource (String resourceName)
  {
    String vorgefertigteXMLStruktur = null;
    
    InputStream is = this.getClass().getResourceAsStream( resourceName );
    if (is == null)
      System.err.println("Resource " + resourceName + " nicht gefunden ");
        
    BufferedInputStream bis = new BufferedInputStream (is, 4096);
    StringBuffer sb = new StringBuffer ();
    
    byte [] buf = new byte [4096];
    
    try
    {
      for (int len; (len = bis.read(buf)) !=-1;)
      {
        sb.append( new String (buf, 0, len) );
      }
    }
    catch (IOException e)
    {
//      e.printStackTrace();
    }

    vorgefertigteXMLStruktur = sb.toString();
    
    return vorgefertigteXMLStruktur;
  }

  /**
   * Methode schreibt einen String in eine Datei
   * @param dateiName Dateiname
   * @param text String der in die Datei geschrieben wird
   */
  public void schreibeInDatei (String dateiName, String text)
  {
    FileOutputStream schreibeStrom = null;
    
    try
    {
      schreibeStrom = new FileOutputStream(dateiName);
      
      for (int i=0; i < text.length(); i++)
        schreibeStrom.write((byte)text.charAt(i));

      schreibeStrom.close();
    }
    catch (FileNotFoundException e)
    {
      e.printStackTrace();
    }
    
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  /**
   * Methode prft ob eine XML-Datei (aus einer Resource) korrekt eingelesen wird, im Fehlerfall wird der
   * Fehlertext zurckgeliefert
   * @param resource  Resource mit XML-Datei
   * @return im Fehlerfall der Fehlertext, sonst null
   */
  private String pruefeXMLString (String resource)
  {
    String datei    = "testLeseRootElement.xml";
    
    String xml = einlesenAusResource (resource);
    schreibeInDatei( datei, xml );
    
    XMLTools xt = new XMLTools ();
    
    Element r = xt.leseRootElement( datei );
    if (xt.isFehler())
      return (xt.getFehlerText());
    
    return null;
  }
}
